#include <Arduino.h>
#include "ac101.h"
#include "driver/i2s.h"
#include "esp_task_wdt.h"
#include "TFT_eSPI.h"
#include "arduinoFFT.h"

arduinoFFT FFT = arduinoFFT();

TFT_eSPI tft = TFT_eSPI();
TFT_eSprite vumeter = TFT_eSprite(&tft);
TFT_eSprite waveform_left = TFT_eSprite(&tft);
TFT_eSprite waveform_right = TFT_eSprite(&tft);

#define spectrumWidth 240
#define spectrumHeight 90
#define waveWidth 128
#define waveHeight 90

#define SAMPLE_RATE     44100
#define I2S_NUM         0
#define I2S_BCK_IO      27
#define I2S_WS_IO       26
#define I2S_DO_IO       25
#define I2S_DI_IO       35
#define CODEC_SCK       32
#define CODEC_SDA       33

//audio processing frame length in samples (L+R) 64 samples (32R+32L) 256 Bytes
#define FRAMELENGTH    512
//sample count per channel for each frame (32)
#define SAMPLECOUNT   FRAMELENGTH / 2
//channel count inside a frame (always stereo = 2)
#define CHANNELCOUNT  2
//frame size in bytes
#define FRAMESIZE   FRAMELENGTH * 4
//audio processing priority
#define AUDIO_PROCESS_PRIORITY  10

//dma buffer length 32 bytes (8 samples: 4L+4R)
#define DMABUFFERLENGTH 32
//dma buffer count 20 (640 Bytes: 160 samples: 80L+80R) 
#define DMABUFFERCOUNT  20

//codec instance
AC101 codec;

//buffers
float wleft[] = {0,0};
float wright[] = {0,0};
int32_t inbuffer[FRAMELENGTH];
int32_t outbuffer[FRAMELENGTH];
float inleft[FRAMESIZE];
float inright[FRAMESIZE];
float outleft[FRAMESIZE];
float outright[FRAMESIZE];
unsigned long processedframes;

double vReal[FRAMESIZE];
double vImag[FRAMESIZE];
int32_t sampleIn;
int peak[8];
//DIGITAL SIGNAL PROCESSING FUNCTION
void dsprocess(float* left_in, float* right_in, float* left_out, float* right_out, int samplecount)
{
  //BEGIN WRITE YOUR DIGITAL PROCESSING ALGORITHM HERE
  for(unsigned int i=0;i<samplecount;i++)
  {
    sampleIn = 0;
    left_out[i] = left_in[i]/4;
    right_out[i] = right_in[i]/4;
    sampleIn =  left_out[i];
    sampleIn >>=8;
    vReal[i] = sampleIn;
    vImag[i] = 0;   
  }
  //END WRITE YOUR DIGITAL PROCESSING ALGORITHM HERE
}

void i2s_task(void* arg)
{
  size_t bytesread, byteswritten;
  processedframes = 0;

  //initialize all output buffer to zero
  for(int i= 0; i< FRAMELENGTH; i++)
    outbuffer[i] = 0;
  for(int i=0; i< SAMPLECOUNT; i++)
  {
    outleft[i]=0;
    outright[i]=0;
  }
  
  while(true)
  {
    processedframes++;
    i2s_read((i2s_port_t)I2S_NUM,(void*) inbuffer, FRAMESIZE, &bytesread, 20);
    //convert from int to float
    for(int i=0,k=0;k<SAMPLECOUNT;k++,i+=2)
    {
      inleft[k] = (float) inbuffer[i];  
      inright[k] = (float) inbuffer[i+1];
      //inright[k] = inleft[k];
    }
  
    //process the signal
    dsprocess(inleft, inright, outleft, outright, SAMPLECOUNT);
    
    //convert back float to int
    for(int i=0,k=0;k<SAMPLECOUNT;k++,i+=2)
    {
      outbuffer[i] = (int32_t) outleft[k];
      outbuffer[i+1] = (int32_t) outright[k];
    }
    i2s_write((i2s_port_t)I2S_NUM,(void*) outbuffer, FRAMESIZE, &byteswritten, 20);
    esp_task_wdt_reset();
  }
  vTaskDelete(NULL);
}

void i2s_setup()
{
  i2s_config_t i2s_config;
  i2s_config.mode =(i2s_mode_t) (I2S_MODE_MASTER | I2S_MODE_TX | I2S_MODE_RX);
  i2s_config.sample_rate = SAMPLE_RATE;
  i2s_config.bits_per_sample = (i2s_bits_per_sample_t) 32;
  i2s_config.channel_format = I2S_CHANNEL_FMT_RIGHT_LEFT;
  i2s_config.communication_format = (i2s_comm_format_t)(I2S_COMM_FORMAT_I2S | I2S_COMM_FORMAT_I2S_MSB);
  i2s_config.dma_buf_count = DMABUFFERCOUNT;
  i2s_config.dma_buf_len = DMABUFFERLENGTH;
  i2s_config.use_apll = false;
  i2s_config.intr_alloc_flags = ESP_INTR_FLAG_LEVEL1 ;
  
  i2s_pin_config_t pin_config;
  pin_config.bck_io_num = I2S_BCK_IO;
  pin_config.ws_io_num = I2S_WS_IO;
  pin_config.data_out_num = I2S_DO_IO;
  pin_config.data_in_num = I2S_DI_IO;

  i2s_driver_install((i2s_port_t)I2S_NUM, &i2s_config, 0, NULL);
  i2s_set_pin((i2s_port_t)I2S_NUM, &pin_config);
  i2s_set_clk((i2s_port_t)I2S_NUM, SAMPLE_RATE, (i2s_bits_per_sample_t) 32, I2S_CHANNEL_STEREO);
}

void setup() 
{
  Serial.begin(115200);
  tft.init();
  tft.setRotation(3);
  tft.fillScreen(TFT_BLUE);
  tft.setTextColor(TFT_WHITE, TFT_BLUE);
  tft.setTextSize(1);
  tft.setCursor(60, 230);
  tft.print("left");
  tft.setCursor(225, 230);
  tft.print("right");
  vumeter.createSprite(spectrumWidth, spectrumHeight);
  vumeter.setColorDepth(8);
  waveform_left.createSprite(waveWidth, waveHeight);
  waveform_left.setColorDepth(8);
  waveform_right.createSprite(waveWidth, waveHeight);
  waveform_right.setColorDepth(8);
  //codec setup
  codec.setup(CODEC_SDA, CODEC_SCK);
  /*
  //enable line input on both left and right channels
  codec.LeftLineEnable(true);
  codec.RightLineEnable(true);
  */
  //enable microphone on both left and right channels
  codec.LeftMicEnable(true);
  codec.RightMicEnable(true);
  //codec.SetMicGain(1);
  Serial.println(codec.GetMicGain());

   /*
  //enable microphone on  left channel and line input on right channel
  codec.LeftMicEnable(true);
  codec.RightLineEnable(true);
  */
  
  i2s_setup();
  xTaskCreate(i2s_task, "i2s_task", 4096, NULL, AUDIO_PROCESS_PRIORITY, NULL);
}

void loop() 
{
  waveform_left.fillSprite(TFT_BLACK);
  waveform_right.fillSprite(TFT_BLACK);
  for (uint16_t counter = 1; counter < FRAMESIZE; counter++)
  {
    waveform_left.drawLine(counter - 1,
                           map(outleft[counter - 1], 0, 60000000, 45, waveHeight),
                           counter,
                           map(outleft[counter], 0, 60000000, 45, waveHeight),
                           ILI9341_WHITE);
    waveform_right.drawLine(counter - 1,
                            map(outright[counter - 1], 0, 60000000, 45, waveHeight),
                            counter,
                            map(outright[counter], 0, 60000000, 45, waveHeight),
                            ILI9341_WHITE);
  }
  waveform_left.pushSprite(80 - (waveWidth / 2), 130);
  waveform_right.pushSprite(240 - (waveWidth / 2), 130);

  vumeter.fillScreen( TFT_BLACK );
  
  FFT.Windowing( vReal, SAMPLECOUNT, FFT_WIN_TYP_HAMMING, FFT_FORWARD) ;
  FFT.Compute( vReal, vImag, SAMPLECOUNT, FFT_FORWARD );
  FFT.ComplexToMagnitude( vReal, vImag, SAMPLECOUNT );
  for (int i = 0; i < 8; i++) {
    peak[i] = 0;
  }  
  for (int i = 2; i < (SAMPLECOUNT); i++){ 
      if (i<=3 )            peak[0] = max(peak[0], (int)(vReal[i])); // 125Hz
      if (i >3   && i<=6  ) peak[1] = max(peak[1], (int)(vReal[i])); // 250Hz
      if (i >6   && i<=13 ) peak[2] = max(peak[2], (int)(vReal[i])); // 500Hz
      if (i >13  && i<=27 ) peak[3] = max(peak[3], (int)(vReal[i])); // 1000Hz
      if (i >27  && i<=55 ) peak[4] = max(peak[4], (int)(vReal[i])); // 2000Hz
      if (i >55  && i<=112) peak[5] = max(peak[5], (int)(vReal[i])); // 4000Hz
      if (i >112 && i<=229) peak[6] = max(peak[6], (int)(vReal[i])); // 8000Hz
      if (i >229          ) peak[7] = max(peak[7], (int)(vReal[i])); // 16000Hz
  }
 for (int i = 0; i < 8; i++)
  {
    vumeter.fillRect(i * 30 + 5,
                     90 - map(peak[i],0,20000000,0,90),
                     20,
                     map(peak[i],0,20000000,0,90),
                     TFT_GREEN);
    peak[i] = 0;
  }
vumeter.pushSprite(40, 20);  
}
 